@@ -53,9 +53,8 @@ struct btrfs_mapping_tree {
 
 static inline unsigned long btrfs_chunk_item_size(int num_stripes)
 {
-	BUG_ON(num_stripes == 0);
-	return sizeof(struct btrfs_chunk) +
-		sizeof(struct btrfs_stripe) * (num_stripes - 1);
+	return offsetof(struct btrfs_chunk, stripe) +
+		sizeof(struct btrfs_stripe) * num_stripes;
 }
 
 static inline u32 __BTRFS_LEAF_DATA_SIZE(u32 nodesize)
@@ -100,7 +99,8 @@ static inline u32 __BTRFS_LEAF_DATA_SIZE(u32 nodesize)
 	 BTRFS_FEATURE_INCOMPAT_ZONED |			\
 	 BTRFS_FEATURE_INCOMPAT_EXTENT_TREE_V2 |	\
 	 BTRFS_FEATURE_INCOMPAT_RAID_STRIPE_TREE |	\
-	 BTRFS_FEATURE_INCOMPAT_SIMPLE_QUOTA)
+	 BTRFS_FEATURE_INCOMPAT_SIMPLE_QUOTA |		\
+	 BTRFS_FEATURE_INCOMPAT_REMAP_TREE)
 #else
 #define BTRFS_FEATURE_INCOMPAT_SUPP			\
 	(BTRFS_FEATURE_INCOMPAT_MIXED_BACKREF |		\
@@ -293,6 +293,8 @@ struct btrfs_block_group {
 	bool zone_is_active;
 
 	u64 global_root_id;
+	u64 remap_bytes;
+	u32 identity_remap_count;
 };
 
 struct btrfs_device;
@@ -308,6 +310,7 @@ struct btrfs_fs_info {
 	struct btrfs_root *uuid_root;
 	struct btrfs_root *block_group_root;
 	struct btrfs_root *stripe_root;
+	struct btrfs_root *remap_root;
 
 	struct rb_root global_roots_tree;
 	struct rb_root fs_root_tree;
@@ -673,6 +676,10 @@ static inline u32 BTRFS_MAX_XATTR_SIZE(const struct btrfs_fs_info *info)
 
 #define BTRFS_RAID_STRIPE_KEY	230
 
+#define BTRFS_IDENTITY_REMAP_KEY	234
+#define BTRFS_REMAP_KEY			235
+#define BTRFS_REMAP_BACKREF_KEY		236
+
 #define BTRFS_BALANCE_ITEM_KEY	248
 
 /*
