@@ -1,6 +1,6 @@
 #!/bin/bash
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
 
 check_prereq btrfs-corrupt-block
 check_prereq mkfs.btrfs
